/* 
 * $TSUKUBA_Release: Omni XcalableMP Compiler Software Version 0.5 (alpha) $
 * $TSUKUBA_Copyright:
 *  Copyright (C) 2010 University of Tsukuba
 *  
 *  This software is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License version
 *  2.1 published by the Free Software Foundation.
 *  
 *  Please check the Copyright and License information in the file named
 *  COPYRIGHT in the top directory of the Omni Compiler Software release kit.
 *  $
 */
package xcodeml.util;

import xcodeml.XmObj;

/**
 * Exception about XcodeML description.
 */
public class XmBindingException extends RuntimeException
{
    private static final long serialVersionUID = 1482673969593630141L;
    
    private XmObj _xobj;

    /**
     * Creates XbcBindingException.
     *
     * @param msg the detail message
     * @param xobj a object binding to XcodeML which has fault description.
     */
    public XmBindingException(XmObj xobj, String msg)
    {
        super(msg);
        _xobj = xobj;
    }

    /**
     * Creates XbcBindingException.
     *
     * @param e the cause
     * @param xobj a object binding to XcodeML which has fault description
     */
    public XmBindingException(XmObj xobj, Exception e)
    {
        super(e);
        _xobj = xobj;
    }

    /**
     * Gets a binding object.
     *
     * @return a binding object.
     */
    public XmObj getXbObj()
    {
        return _xobj;
    }

    /**
     * Gets a XcodeML description of a binding object.
     * 
     * @return a XcodeML description
     */
    public String getElementDesc()
    {
        if(_xobj == null)
            return null;

        StringBuffer b = new StringBuffer(128);

        _xobj.makeTextElement(b);
        if(b.length() > 256) {
            b.delete(253, b.length());
            b.append("...");
        }

        return b.toString();
    }
}
