/*
 * $TSUKUBA_Release: Omni XcalableMP Compiler Software Version 0.5 (alpha) $
 * $TSUKUBA_Copyright:
 *  Copyright (C) 2010 University of Tsukuba
 *  
 *  This software is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License version
 *  2.1 published by the Free Software Foundation.
 *  
 *  Please check the Copyright and License information in the file named
 *  COPYRIGHT in the top directory of the Omni Compiler Software release kit.
 *  $
 */

package exc.xcalablemp;

import exc.object.*;
import java.util.Iterator; 

public class XMPrealloc implements XobjectDefVisitor {
  private XMPglobalDecl _globalDecl;
  private XMPobjectTable _globalObjectTable;

  public XMPrealloc(XMPglobalDecl globalDecl) {
    _globalDecl = globalDecl;
    _globalObjectTable = globalDecl.getGlobalObjectTable();
  }

  public void doDef(XobjectDef def) {
    try {
      realloc(def);
    } catch (XMPexception e) {
      // FIXME get line number
      XMP.error(e.getMessage());
    }
  }

  public void realloc(XobjectDef def) throws XMPexception {
    if (def.isVarDecl()) {
      String varName = def.getName();
      XMPalignedArray alignedArray = _globalObjectTable.getAlignedArray(varName);
      if (alignedArray != null) {
        if (alignedArray.realloc()) {
          XobjList allocFuncArgs = Xcons.List(alignedArray.getAddrId().getAddr(), alignedArray.getDescId().Ref());
          for (int i = alignedArray.getDim() - 1; i >= 0; i--) {
            allocFuncArgs.add(Xcons.Cast(Xtype.unsignedlonglongType,
                                         alignedArray.getAccIdAt(i).getAddr()));
          }

          if (alignedArray.getAddrId().getStorageClass() == StorageClass.EXTERN) {
            _globalDecl.addGlobalInitFuncCall("_XCALABLEMP_init_array_alloc_params", allocFuncArgs);
          }
          else {
            _globalDecl.addGlobalInitFuncCall("_XCALABLEMP_alloc_array", allocFuncArgs);
          }

          def.setDef(null);
        }
        else {
          XobjList allocFuncArgs = Xcons.List(alignedArray.getAddrId().getAddr(),
                                              alignedArray.getArrayId().Ref(),
                                              alignedArray.getDescId().Ref());
          for (int i = alignedArray.getDim() - 1; i >= 0; i--) {
            allocFuncArgs.add(Xcons.Cast(Xtype.unsignedlonglongType,
                                         alignedArray.getAccIdAt(i).getAddr()));
          }

          _globalDecl.addGlobalInitFuncCall("_XCALABLEMP_init_array_addr", allocFuncArgs);
        }
      }
    }
  }
}
